#vim: set fileencoding:utf-8

require_relative "liblzma.so"

module LZMA
    module_function
    def encode(data, *args)
        s = Stream.encoder(Filter.lzma2(*args))
        s.update(data, LZMA::FINISH)
    end

    module_function
    def decode(data, *args)
        s = Stream.auto_decoder(*args)
        s.update(data, LZMA::FINISH)
    end

    module_function
    def raw_encode(data, *args)
        s = Stream.raw_encoder(*args)
        s.update(data, LZMA::FINISH)
    end

    module_function
    def raw_decode(data, *args)
        s = Stream.raw_decoder(*args)
        s.update(data, LZMA::FINISH)
    end

    class Stream
        def self.encoder(*args)
            Encoder.new(*args)
        end

        def self.decoder(*args)
            Decoder.new(*args)
        end

        def self.auto_decoder(*args)
            AutoDecoder.new(*args)
        end

        def self.raw_encoder(*args)
            RawEncoder.new(*args)
        end

        def self.raw_decoder(*args)
            RawDecoder.new(*args)
        end
    end

    module Utils
        class CRC32 < Struct.new(:state, :init)
            def initialize(state = 0)
                state = state.to_i
                super(state, state)
            end

            def update(data)
                self.state = Utils.crc32(data, state)
                self
            end

            alias << update

            def finish
                self
            end

            def reset
                self.state = init
                self
            end

            def digest
                [state].pack("N")
            end

            def hexdigest
                "%08x" % state
            end

            alias to_s hexdigest

            def to_str
                "CRC32 <#{hexdigest}>"
            end

            alias inspect to_str
        end

        class CRC64 < Struct.new(:state, :init)
            def initialize(state = 0)
                state = state.to_i
                super(state, state)
            end

            def update(data)
                self.state = Utils.crc64(data, state)
                self
            end

            alias << update

            def finish
                self
            end

            def reset
                self.state = init
                self
            end

            def digest
                [state].pack("N")
            end

            def hexdigest
                "%016x" % state
            end

            alias to_s hexdigest

            def to_str
                "CRC64 <#{hexdigest}>"
            end

            alias inspect to_str
        end
    end
end
